// This may look like C code, but it is really -*- C++ -*-
//
// This class is to hold one set of sample voltages for all saved
// nodes (for HB, EHB, etc.) . It should be as light as possible since
// one of these is required for each sweep/time sample.
//
// by Carlos E. Christoffersen

#ifndef DVectorHolder_h
#define DVectorHolder_h 1

#include <vector>
#include <string>
#include <cassert>
#include <cstdio>

class DVectorHolder : public SampleHolder
{
public:

  DVectorHolder(const int& numnodes, const int& numsamples,
		double* timebase) 
  {
    assert(numnodes > 0 && numsamples > 0);
    uvec = new double [numnodes*numsamples];
    // Seems wasteful to store one time base per sample, but if we
    // allow variable resolution this will be necessary anyway.
    this->numsamples = numsamples;
    time2 = new double [numsamples];
    // Use pointer arithmetic for efficiency (may not be necessary)
    double* pend = time2+numsamples;
    for (double* p(time2); p < pend; p++,timebase++)
      *p = *timebase;
  }
  
  virtual ~DVectorHolder() 
  {
    delete [] time2;
    delete [] uvec;
  }

  // Write node(index) information to out string
  virtual void writeNode(const int& index, std::string& out)
  {
    char buffer[256];
    out = "";
    // Print in xyz format (x: main time, y: periodic time, z: voltage)
    for (int i(0); i < numsamples; i++) {
      sprintf(buffer,"%g\t %g\t %g\n", this->getVal(), time2[i], 
	      uvec[index*numsamples+i]);
      out += buffer;
    }
  }

  // Set values. 
  inline void saveValue(const int& index, double* vec)
  {
    // Use pointer arithmetic for efficiency
    double* pend = uvec+numsamples*(index+1);
    for (double* p(uvec+numsamples*index); p < pend; p++,vec++)
      *p = *vec;
  }

private:
  
  double* uvec;
  double* time2;
  int numsamples;

};


#endif


