// This may look like C code, but it is really -*- C++ -*-
//
// Test TimeMarching analysis. For now do something simple to just to test the
// concept.
//
// 
// by Carlos E. Christoffersen

#ifndef TimeMarching_H
#define TimeMarching_H

#include <carrot/Circuit.h>
#include <carrot/UMFPackDI.h>

template <typename IntegMethod>
class TimeMarching
{
public:

  TimeMarching(Circuit* cir, const int& nrcs, IntegMethod* imo,
	       UMFPackDI& nam);

  ~TimeMarching();

  // Run one time step. The integration method object (imo) is
  // supposed to be correctly initialised. The user is responsible to
  // call advance() in the integration method.
  // uvec[-1] and svec[-1] are assumed to be allocated.
  // t is the starting time (current time is t+tstep).
  bool step(UMFPackDI& nam, double* uvec, double* q, double* svec, 
	    const double& t, const double& tstep, const bool& newtstep,
	    double& residual, int& iterations);

  int fillNAMandRHS(UMFPackDI& nam, double* svec, double* uvec_j, 
		     const double& beta, const double& gmin);

private:

  Circuit* cir;
  int nrcs;
  IntegMethod* imo;
  LinVCCSVector linvccs;
  GenVCQSVector genvcqs;
  LinVCQSVector linvcqs;
  GenVCCSVector genvccs;
  IndepCSVector indvccs;
  double *indep_svec;
  double *dij, *dqj, *vj, *q;

};

#endif
