#include "global.h"
#include "ACCS.h"
#include <carrot/FADBADIndependentCS.h>
// Debug
//#include <cstdio>

static ItemInfo ACCS_info = {
  "accs",
  "AC Current Source",
  "Carlos Christoffersen",
  DEFAULT_ADDRESS
};

// ***********************************************************************
// Double parameters. We need a way to correlate this with the
// information in SpiceDiode_dparams
// ***********************************************************************
enum {AMPLITUDE, FREQ, PHASE, MAX_DPARM_INDEX}; 

static ParamHandler<double>::ParmInfo ACCS_dparams[] = {
  {"amplitude", "Amplitude of current (A)", true, 0.},
  {"freq", "Frequency (Hz)", false, 1.},
  {"phase", "Start phase (rad)", false, 0.}
};

// ***********************************************************************
// Integer parameters
// ***********************************************************************
// enum {TPORT, MAX_IPARM_INDEX};
// static ParamHandler<int>::ParmInfo SpiceDiode_iparams[] = {
//   {"tport", "Use thermal port (4 terminals)", false, 0}
// };


extern "C" {

  //Export the name of the module we are using

  const char * ModuleName(){
    return ACCS_info.type.c_str();
  }

  ParmSet * newParmSet(const char * model){
    return new ParmSet(model, &ACCS_info, 
		       ACCS_dparams, MAX_DPARM_INDEX);
  }

  void delParmSet(ParmSet * p){
    delete p;
    p = 0;
  }

  MacroElement * newObject(ParmSet * p, const char * name){
    return new ACCS(p, name);
  }

  void delObject(MacroElement * e){
    delete e;
    e = 0;
  }

};


// ***********************************************************************
// Internal CS element definitions:
// ***********************************************************************
// This has to be included after enum (for double params) definition

#include "InternalACCS.h"

// Instantiate template
//template InternalACCS;

// ***********************************************************************
// SpiceDiode members start:
// ***********************************************************************
ACCS::ACCS(ParmSet *pset, const std::string& name) : MacroElement(pset, name, 2)
{
}

ACCS::~ACCS()
{
  // There is not much to delete here because the circuit takes care
  // of deleting the CSs. (Is this right?)
}

void ACCS::init()
{
  double *dparm = getParmSet()->getDHandler()->getParmVec();
  // This flow should be similar for many MacroElements:
  //
  // 0. Check parameters and terminals
  // (We should really throw and exception in the following:)
  getParmSet()->checkParms();
  assert(checkTerms());
  // No internal nodes needed for now
  setNInternalNodes(0);
  // 1. Create individual CSs
  InternalACCS* accs = new InternalACCS(dparm, NULL);
  // 2. connect terminals.
  static const int tvec[2] = {0,1};
  insert(accs, tvec);

}

