// This may look like C code, but it is really -*- C++ -*-
//
// InternalACCS is a sinusoidal current source
//
//          1  
//          o        
//          |       
//          |       
//         (^) J
//          |       
//          |     
//          o       
//          2  
//
//
// by Carlos E. Christoffersen

#ifndef InternalACCS_h
#define InternalACCS_h 1

class InternalACCS_eval {
public:
  // This are locally-defined variables (not needed)
  enum {OMEGA, MAX_CVAR_INDEX};

  InternalACCS_eval(int *iparm) {} // iparm not needed here

  // Main evaluation function. Only voltage dependence considered here.
  template <typename currDouble, typename paramDouble, typename cvarDouble>
  inline void operator()(currDouble& iout, const double& ctime, 
			 paramDouble* dparm, cvarDouble* cvar)
  {
    iout = dparm[AMPLITUDE] * cos(cvar[OMEGA] * ctime + dparm[PHASE]);
  }

  // For efficiency, we may want to pre-calculate some expressions
  // that do not change with voltage or temperature.  Adjust variables
  // to a new parameer set.
  template <typename varDouble>
  inline void newParms(varDouble* cvar, varDouble* dparm)
  {
    cvar[OMEGA] = dparm[FREQ] * twopi;
  }

  //private:
  
  // Pointer to integer parameter vector from macroelement
  //  int *iparm; // not necessary for this model

};

typedef FADBADIndependentCS<InternalACCS_eval, MAX_DPARM_INDEX> InternalACCS;


#endif


