// This may look like C code, but it is really -*- C++ -*-
//
// Implements the BC junction currents of a NPN BJT
//
//             
// 

#ifndef BJTBC_H
#define BJTBC_H 1

class BJTBC_eval {
public:
  // This are locally-defined variables
  enum {T_ISoBR, T_ISC, VT, MAX_TVAR_INDEX};
  enum {dummy, MAX_CVAR_INDEX};

  BJTBC_eval(int *iparm) {} // iparm not needed here

  // Main evaluation function. Only voltage dependence considered here.
  template <typename currDouble, typename voltDouble, typename paramDouble, 
	    typename varDouble, typename cvarDouble>
  inline void operator()(currDouble& ibc, voltDouble* v, 
			 paramDouble* dparm, 
			 varDouble* tvar, cvarDouble* cvar)
  {
    currDouble ibr_br;
    getIexp(ibr_br, v[0], tvar[T_ISoBR], dparm[NR], tvar[VT]);
    currDouble ilc;
    getIexp(ilc, v[0], tvar[T_ISC], dparm[NC], tvar[VT]);
    ibc = ibr_br + ilc;
    ibc *= dparm[AREA];
  }

  // Adjust variables to a new temperature.
  template <typename tvarDouble, typename varDouble, typename tempDouble>
  inline void setTemp(tvarDouble* tvar, 
		      varDouble* dparm, varDouble* cvar, tempDouble& T)
  {
    tempDouble Tabs = T + tzero;
    // Parameter calculation 
    getVt(tvar[VT], Tabs);
    getIs(tvar[T_ISoBR], dparm[IS], dparm[NF], tvar[VT], dparm, Tabs);
    getIs(tvar[T_ISC], dparm[ISC], dparm[NC], tvar[VT], dparm, Tabs);
    tvarDouble br;
    getB(br, dparm[BR], dparm, Tabs);
    tvar[T_ISoBR] /= br;
  }
  
  // For efficiency, we may want to pre-calculate some expressions
  // that do not change with voltage or temperature.  Adjust variables
  // to a new parameter set.
  template <typename varDouble>
  inline void newParms(varDouble* cvar, varDouble* dparm)
  {
    // Do nothing.
  }

  // Calculate power. pout initially contains the current.
  template <typename powerDouble, typename voltDouble, 
	    typename paramDouble>
  inline void power(powerDouble& pout, voltDouble* v, 
		    paramDouble* dparm)
  {
    pout *= v[0];
  }

  //private:
  
  // Pointer to integer parameter vector from macroelement
  //  int *iparm; // not necessary for this model

};

// No thermal port
typedef FADBADGenericVCS<BJTBC_eval, GenericVCCS, 1, 
			 MAX_DPARM_INDEX, TEMP, false> BJTBC;
// With thermal port
typedef FADBADTGenericVCS<BJTBC_eval, GenericVCCS, 2, 
			 MAX_DPARM_INDEX, TEMP> TBJTBC;

// Power source
typedef FADBADPowerSource<BJTBC_eval, 2, 
 			  MAX_DPARM_INDEX, TEMP> BJTBC_PS;

#endif
