// This may look like C code, but it is really -*- C++ -*-
//
// Implements the charge stored in the BC junction
//
//             
//          o  1              
//          |                            
//        --+--
//               QBC
//        --+--
//          |                          
//          o  0    	      
//                
// 

#ifndef QBC_H
#define QBC_H 1

class QBC_eval {
public:
  // This are locally-defined variables
  enum {T_IS, VT, T_VJC, T_CJC, k5, k6, k7, MAX_TVAR_INDEX};
  enum {k4, EGAPn, MAX_CVAR_INDEX};

  QBC_eval(int *iparm) {} // iparm not needed here

  // Main evaluation function. Only voltage dependence considered here.
  template <typename currDouble, typename voltDouble, typename paramDouble, 
	    typename varDouble, typename cvarDouble>
  inline void operator()(currDouble& qbc, voltDouble* v, 
			 paramDouble* dparm, 
			 varDouble* tvar, cvarDouble* cvar)
  {
    // Depletion charge
    getQj(qbc, v[0], tvar[T_VJC], tvar[T_CJC], 
	  tvar[k5], tvar[k6], tvar[k7], dparm[MJC], dparm[FC], cvar[k4]);
    qbc *= dparm[XCJC];
    if (dparm[TR] > 0.) {
      // Need to calculate ibr for TT component
      currDouble ibr;
      getIexp(ibr, v[0], tvar[T_IS], dparm[NR], tvar[VT]);
      qbc += dparm[TR] * ibr;
    }
    // Area effect
    qbc *= dparm[AREA];
  }

  // Adjust variables to a new temperature.
  template <typename tvarDouble, typename varDouble, typename tempDouble>
  inline void setTemp(tvarDouble* tvar, 
		      varDouble* dparm, varDouble* cvar, tempDouble& T)
  {
    tempDouble Tabs = T + tzero;
    // Parameter calculation 
    getVt(tvar[VT], Tabs);
    getIs(tvar[T_IS], dparm[IS], dparm[NF], tvar[VT], dparm, Tabs);
    getVbi(tvar[T_VJC], tvar[VT], cvar[EGAPn], dparm[VJC], dparm, Tabs);
    getCbi(tvar[T_CJC], tvar[VT], 
	   tvar[T_VJC], dparm[VJC],
	   dparm[CJC], dparm[MJC], dparm, Tabs);
    getk567(tvar[k5], tvar[k6], tvar[k7], 
	    tvar[T_VJC], tvar[T_CJC],
	    dparm[MJC], dparm[FC], cvar[k4]);
  }
  
  // For efficiency, we may want to pre-calculate some expressions
  // that do not change with voltage nor temperature.  Adjust variables
  // to a new parameer set.
  template <typename varDouble>
  inline void newParms(varDouble* cvar, varDouble* dparm)
  {
    cvar[k4] = 1. - dparm[MJC];
    varDouble tnom = dparm[TNOM]+tzero;
    getEgap(cvar[EGAPn], dparm, tnom);
  }

  //private:
  
  // Pointer to integer parameter vector from macroelement
  //  int *iparm; // not necessary for this model

};


typedef FADBADGenericVCS<QBC_eval, GenericVCQS, 1,
			 MAX_DPARM_INDEX, TEMP, false> QBC;
// With thermal port
typedef FADBADTGenericVCS<QBC_eval, GenericVCQS, 2, 
			  MAX_DPARM_INDEX, TEMP> TQBC;

#endif
