#include "Inductor.h"
#include <carrot/Options.h>

static ItemInfo Inductor_info = {
  "ind",
  "Linear Inductor",
  "Carlos Christoffersen"
  "http://vision.lakeheadu.ca"
};

// Double Parameters
// Make sure enum and ParInfo definitions are in the same order!
enum {IND, RINT, MAX_DPARM_INDEX};

//Insert ParmInfo values here
static ParamHandler<double>::ParmInfo Inductor_dparams[] = { 
  {"l", "Inductance (H)", true, 0.},
  {"rint", "Internal resistance (Ohms", false, 0.}
};

// Uncomment for integer parameters
//enum {MAX_IPARM_INDEX};
//static ParamHandler<int>::ParmInfo Inductor_iparams[] = {  };


//Library loading mechanism
extern "C" {
  const char * ModuleName(){
    return Inductor_info.type.c_str();
  }
  ParmSet * newParmSet(const char * model){
    return new ParmSet(model,
		         &Inductor_info,
		         Inductor_dparams, MAX_DPARM_INDEX); 
//		         Inductor_iparams, MAX_IPARM_INDEX);
  }
  void delParmSet(ParmSet * p){
    delete p;
    p=0;
  }
  MacroElement * newObject(ParmSet * p, const char * name){
    return new Inductor(p, name);
  }
  void delObject(MacroElement * e){
    delete e;
    e = 0;
  }
};

#include "LType.h"
#include "RIType.h"

Inductor::Inductor(ParmSet *pset, const std::string& name)
  :MacroElement(pset, name, 2)
{
  //Constructor code goes here 
}

Inductor::~Inductor()
{
  //Destructor code goes here
}

void Inductor::init()
{
  const double &scale = options.getDHandler()->getParmVec()[OPT_SCALE];
  // access to double parameters
  double *dparm = getParmSet()->getDHandler()->getParmVec();
  // Check required parameters
  getParmSet()->checkParms();
  // Check external terminal connections
  assert(checkTerms());
  // Create things independent of internal resistor
  LinearVCQS* lvcqs = new LType(dparm, NULL);
  static const int tvec_vcqs[4] = {-1,2,-1,2};
  insert(lvcqs, tvec_vcqs);
  // Gyrator sources
  assert(scale != 0.);
  LinearVCCS* g1 = new LinearVCCS(scale);
  LinearVCCS* g2 = new LinearVCCS(scale);
  setNInternalNodes(1);
  static const int tvec_g1[4] = {-1,2,0,1};
  insert(g1, tvec_g1);
  static const int tvec_g2[4] = {0,1,2,-1};
  insert(g2, tvec_g2);
  // Add internal resistor without adding an extra internal node.
  if (dparm[RINT] != 0.) {
    LinearVCCS* rint = new RIType(dparm, NULL);
    static const int tvec_rint[4] = {2,-1,2,-1};
    insert(rint, tvec_rint);
  }
}


