// This may look like C code, but it is really -*- C++ -*-
//
// Implements the DS current of a MESFET using Curtice's cubic model
//
// Important: time delay not implemented. It has to be implemented in
// the simulator in each analysis type (except DC). (adding "delay
// cells"? we have to see the effect of these in derivatives)
//             
// 

#ifndef IdsCurtice_H
#define IdsCurtice_H 1

class IdsCurtice_eval {
public:
  // This are locally-defined variables
  enum {T_IDSFAC, T_VT0, MAX_TVAR_INDEX};
  enum {dummy, MAX_CVAR_INDEX};

  IdsCurtice_eval(int *iparm) {} // iparm not needed here

  // Main evaluation function. Only voltage dependence considered here.
  template <typename currDouble, typename voltDouble, typename paramDouble, 
	    typename varDouble, typename cvarDouble>
  inline void operator()(currDouble& ids, voltDouble* v, 
			 paramDouble* dparm, 
			 varDouble* tvar, cvarDouble* cvar)
  {
    // v[0] : Vgs
    // v[1] : Vds
    if (v[0] > tvar[T_VT0]) {
      currDouble vx = 1. + dparm[BETA] * (dparm[VDS0] - v[1]);
      vx *= v[0]; // Note: time delay not implemented for now
      // Calculate ids with few temporaries (we should do this
      // everywhere)
      ids = vx;
      ids *= dparm[A3];
      ids += dparm[A2];
      ids *= vx;
      ids += dparm[A1];
      ids *= vx;
      ids += dparm[A0];
      ids *= my_tanh(dparm[GAMA] * v[1]);
      ids *= tvar[T_IDSFAC];
      ids *= dparm[AREA];
    } else {
      ids = 0.;
    }
  }

  // Adjust variables to a new temperature.
  template <typename tvarDouble, typename varDouble, typename tempDouble>
  inline void setTemp(tvarDouble* tvar, 
		      varDouble* dparm, varDouble* cvar, tempDouble& T)
  {
    // Parameter calculation 
    tvarDouble Dt = T - dparm[TNOM];
    tvar[T_IDSFAC] = pow(1. + dparm[TM] * Dt, dparm[TME]);
    tvar[T_VT0] = 1. + dparm[AVT0] * Dt + dparm[BVT0] * Dt * Dt;
    tvar[T_VT0] *= dparm[VT0];
  }
  
  // For efficiency, we may want to pre-calculate some expressions
  // that do not change with voltage or temperature.  Adjust variables
  // to a new parameter set.
  template <typename varDouble>
  inline void newParms(varDouble* cvar, varDouble* dparm)
  {
    // Do nothing.
  }

  // Calculate power. pout initially contains the current.
  template <typename powerDouble, typename voltDouble, 
	    typename paramDouble>
  inline void power(powerDouble& pout, voltDouble* v, 
		    paramDouble* dparm)
  {
    pout *= v[1];
  }

  //private:
  
  // Pointer to integer parameter vector from macroelement
  //  int *iparm; // not necessary for this model

};

// No thermal port
typedef FADBADGenericVCS<IdsCurtice_eval, GenericVCCS, 2, 
			 MAX_DPARM_INDEX, TEMP, false> IdsCurtice;
// With thermal port
typedef FADBADTGenericVCS<IdsCurtice_eval, GenericVCCS, 3, 
			  MAX_DPARM_INDEX, TEMP> TIdsCurtice;
// Power source
typedef FADBADPowerSource<IdsCurtice_eval, 3, 
 			  MAX_DPARM_INDEX, TEMP> IdsPS;

#endif
