// This may look like C code, but it is really -*- C++ -*-
//
// Implements the GD current of a MESFET in Curtice's cubic model
// (exactly the same as the GS current source)
//             
// 

#ifndef IgdCurtice_H
#define IgdCurtice_H 1

class IgdCurtice_eval {
public:
  // This are locally-defined variables
  enum {T_IS, T_IB0, VT, MAX_TVAR_INDEX};
  enum {dummy, MAX_CVAR_INDEX};

  IgdCurtice_eval(int *iparm) {} // iparm not needed here

  // Main evaluation function. Only voltage dependence considered here.
  template <typename currDouble, typename voltDouble, typename paramDouble, 
	    typename varDouble, typename cvarDouble>
  inline void operator()(currDouble& igd, voltDouble* v, 
			 paramDouble* dparm, 
			 varDouble* tvar, cvarDouble* cvar)
  {
    // v[0] : Vgs
    getIexp(igd, v[0], tvar[T_IS], tvar[T_IB0], tvar[VT], dparm);
    igd *= dparm[AREA];
  }

  // Adjust variables to a new temperature.
  template <typename tvarDouble, typename varDouble, typename tempDouble>
  inline void setTemp(tvarDouble* tvar, 
		      varDouble* dparm, varDouble* cvar, tempDouble& T)
  {
    tempDouble Tabs = T + tzero;
    // Parameter calculation 
    getVt(tvar[VT], Tabs);
    getIs(tvar[T_IS], tvar[VT], dparm[IS], dparm[N], dparm, Tabs);
    getIs(tvar[T_IB0], tvar[VT], dparm[IB0], dparm[NR], dparm, Tabs);
  }
  
  // For efficiency, we may want to pre-calculate some expressions
  // that do not change with voltage or temperature.  Adjust variables
  // to a new parameter set.
  template <typename varDouble>
  inline void newParms(varDouble* cvar, varDouble* dparm)
  {
    // Do nothing.
  }

  // Calculate power. pout initially contains the current.
  template <typename powerDouble, typename voltDouble, 
	    typename paramDouble>
  inline void power(powerDouble& pout, voltDouble* v, 
		    paramDouble* dparm)
  {
    pout *= v[0];
  }

  //private:
  
  // Pointer to integer parameter vector from macroelement
  //  int *iparm; // not necessary for this model

};

// No thermal port
typedef FADBADGenericVCS<IgdCurtice_eval, GenericVCCS, 1, 
			 MAX_DPARM_INDEX, TEMP, false> IgdCurtice;
// With thermal port
typedef FADBADTGenericVCS<IgdCurtice_eval, GenericVCCS, 2, 
			 MAX_DPARM_INDEX, TEMP> TIgdCurtice;
// Power source
typedef FADBADPowerSource<IgdCurtice_eval, 2, 
 			  MAX_DPARM_INDEX, TEMP> IgdPS;


#endif
