// This may look like C code, but it is really -*- C++ -*-
//
// Implementation of the Source Drain (Isd) current using using Level3 
// Semi-empirical model
//
// Important: time delay not implemented. It has to be implemented in
// the simulator in each analysis type (except DC). (adding "delay
// cells"? we have to see the effect of these in derivatives)
//             
//
#ifndef Isd1L3_H
#define Isd1L3_H 1

class Isd1L3_eval {
public:
  // This are locally-defined variables
  
  enum {VT, PHI, EGFET, MAX_TVAR_INDEX};
  enum {dummy, MAX_CVAR_INDEX};
 
  Isd1L3_eval(int *iparm) {} // iparm not needed here

  // Main evaluation function. Only voltage dependence considered here.
  template <typename currDouble, typename voltDouble, typename paramDouble, 
	    typename varDouble, typename cvarDouble>
  inline void operator()(currDouble& isd1L3, voltDouble* v, paramDouble* dparm, 
  			 varDouble* tvar, cvarDouble* cvar)
  {
    // v[0] = Vsg,  v[1] = Vsd,  v[2] = vsb
    
    // Basic electrical parameter calculation
    
    paramDouble Cox = dparm[EOX]/dparm[TOX];
    paramDouble Leff = dparm[L]-2.*dparm[LD];    
    paramDouble KPp = dparm[U0P]*Cox;
    paramDouble B = KPp*dparm[W]/Leff;
    paramDouble Gama = sqrt(2.*dparm[ES]*eCharge*dparm[NSUBP])/Cox;  
    paramDouble Sigma = dparm[ETA]*8.15e-22/(Cox*Leff*Leff*Leff);
    paramDouble XD = sqrt(2.*dparm[ES]/(eCharge*dparm[NSUBP]));
    paramDouble FN = dparm[ES]*dparm[DELT]*3.141592654/(2.*Cox*dparm[W]);
    
    currDouble phibs;
    currDouble sqphbs;
    currDouble Vthp;
    currDouble Va;
    currDouble Vb;
    currDouble Vdsat;
    currDouble Us;
    currDouble DeltaL;
   
    
    	if (v[1] >= 0. && v[2] <= 0.) {		// Vsd >= 0, Vsb <= 0	
  	
    	phibs = tvar[PHI]-v[2];
    	sqphbs = sqrt(phibs);
    
    	}else if (v[1] < 0. && v[2]-v[1] <= 0.){	// Vsd < 0, Vdb <= 0
    	
    	phibs = tvar[PHI]-(v[2]-v[1]);
    	sqphbs = sqrt(phibs);
    		   
        }else if (v[1] >= 0. && v[2] > 0.){	// Vsd >= 0, Vsb > 0
        
        sqphbs = sqrt(tvar[PHI]/(1.+v[2]/(2.*tvar[PHI])));
        phibs = sqphbs*sqphbs;
        	   
        }else{
        
        sqphbs = sqrt(tvar[PHI]/(1.+(v[2]-v[1])/(2.*tvar[PHI])));
        phibs = sqphbs*sqphbs;
        
        }
   
    currDouble WP = XD*sqphbs;
    currDouble WcovXJ = 0.0631353+0.8013292*(WP/dparm[XJ])-0.011107*(WP/dparm[XJ])*(WP/dparm[XJ]);
    currDouble FS = sqrt(1.-(WP/(dparm[XJ]+WP))*(WP/(dparm[XJ]+WP)));
    	       FS *= (dparm[LD]/dparm[XJ]+WcovXJ);
	       FS -= dparm[LD]/dparm[XJ];
    	       FS *= (-1.)*(dparm[XJ]/Leff);
   	       FS += 1.;
   	       
    currDouble Cd = Gama*FS*sqphbs+FN*phibs;
	       Cd /= 2.*phibs;	
	       
    currDouble n = 1.+eCharge*dparm[NFS]/Cox+Cd;       	              
    currDouble Vbi = (dparm[VT0P]+Gama*sqrt(tvar[PHI]))*(-1.);
    
    
	if (v[1] >= 0){
	
    		Vthp = Vbi-Sigma*(v[1])+Gama*FS*sqphbs+FN*phibs;
    		
    	}else{
    	
    		Vthp = Vbi-Sigma*(-v[1])+Gama*FS*sqphbs+FN*phibs;
    	}
    	
    
    currDouble Von = Vthp+n*tvar[VT];
    currDouble FB = FN+Gama*FS/(4.*sqphbs);
    
    
    
    // Vdsat Calculation
    
	if (v[1] >= 0.){						// Vsd >= 0 (Forward Mode)
    
    		if (dparm[VMAX] <= 0. && v[0] <= Von){		
    	
    			Va= (Von-Vthp)/(1.+FB);
    		
    		}else if (dparm[VMAX] <= 0. && v[0] > Von){
    	
    			Va= (v[0]-Vthp)/(1.+FB);
    		
    		}else if (dparm[VMAX] > 0. && v[0] <= Von){
    		
    			Va= (Von-Vthp)/(1.+FB);
    			Us = dparm[U0P]/(1.+dparm[THET]*(Von-Vthp));
    			Vb = dparm[VMAX]*Leff/Us;
    	
    		}else{
    	
    			Va = (v[0]-Vthp)/(1.+FB);
    			Us = dparm[U0P]/(1.+dparm[THET]*(v[0]-Vthp));
    			Vb = dparm[VMAX]*Leff/Us;
    		}
    		
    	}else{								// Vsd < 0 (Reverse Mode)
    
    		if (dparm[VMAX] <= 0. && (v[0]-v[1]) <= Von){		
    	
    			Va= (Von-Vthp)/(1.+FB);
    		
    		}else if (dparm[VMAX] <= 0. && (v[0]-v[1]) > Von){
    	
    			Va= (v[0]-v[1]-Vthp)/(1.+FB);
    		
    		}else if (dparm[VMAX] > 0. && (v[0]-v[1]) <= Von){
    		
    			Va= (Von-Vthp)/(1.+FB);
    			Us = dparm[U0P]/(1.+dparm[THET]*(Von-Vthp));
    			Vb = dparm[VMAX]*Leff/Us;
    			
    		}else{  
    			Va = (v[0]-v[1]-Vthp)/(1.+FB);
    			Us = dparm[U0P]/(1.+dparm[THET]*(v[0]-v[1]-Vthp));
    			Vb = dparm[VMAX]*Leff/Us;}
    }
    
    	if (dparm[VMAX] <= 0.){						
    	
    		Vdsat = Va;						
    	
    	}else{
    	
    		Vdsat = Va+Vb-sqrt(Va*Va+Vb*Vb);
  
    	}

    	
// Drain Current Equation Calculatoin 
   
   
	if (v[1] >= 0.){    	//Forward mode
    
    		if (dparm[VMAX] <= 0.){
    
    			if (v[0] <= Von){	// Weak Inversion

				if (v[1] == 0.){	// Cutoff Region
        
					isd1L3=0.;
	
				}else if (v[1] <= Vdsat){	// Linear Region
				
					DeltaL = sqrt(dparm[KAPA]*XD*XD*Vdsat/8.);
				   	DeltaL *= (v[1]/Vdsat)*(v[1]/Vdsat)*(v[1]/Vdsat);

					if (DeltaL > 0.5*Leff){

						DeltaL = Leff-Leff*Leff/(4*DeltaL);
						isd1L3 = safe_exp((v[0]-Von)/(n*tvar[VT]));
						isd1L3 *= B*(Von-Vthp-(1.+FB)*v[1]/2.)*v[1];
						isd1L3 /= 1.+dparm[THET]*(Von-Vthp);
						isd1L3 /= 1.+Us*v[1]/(Leff*dparm[VMAX]);
						isd1L3 *= Leff/(Leff-DeltaL);
						

					}else{

						isd1L3 = safe_exp((v[0]-Von)/(n*tvar[VT]));
						isd1L3 *= B*(Von-Vthp-(1.+FB)*v[1]/2.)*v[1];
						isd1L3 /= 1.+dparm[THET]*(Von-Vthp);
						isd1L3 /= 1.+Us*v[1]/(Leff*dparm[VMAX]);
						isd1L3 *= Leff/(Leff-DeltaL);				
		
					}
		
					
				}else{		// Saturation region
				
					DeltaL = sqrt(dparm[KAPA]*XD*XD*(v[1]-Vdsat+Vdsat/8.));

					if (DeltaL > 0.5*Leff) {
			
						DeltaL = Leff-Leff*Leff/(4*DeltaL);
						isd1L3 = safe_exp((v[0]-Von)/(n*tvar[VT]));
						isd1L3 *= B*(Von-Vthp-(1.+FB)*Vdsat/2.)*Vdsat;
						isd1L3 /= (1.+dparm[THET]*(Von-Vthp));
						isd1L3 /= 1.+Us*Vdsat/(Leff*dparm[VMAX]);
						isd1L3 *= Leff/(Leff-DeltaL);
						
					}else{
			
						isd1L3 = safe_exp((v[0]-Von)/(n*tvar[VT]));
						isd1L3 *= B*(Von-Vthp-(1.+FB)*Vdsat/2.)*Vdsat;
						isd1L3 /= (1.+dparm[THET]*(Von-Vthp));
						isd1L3 /= 1.+Us*Vdsat/(Leff*dparm[VMAX]);
						isd1L3 *= Leff/(Leff-DeltaL);
								
					}
				
				}
    
    			}else{		// Strong Inversion
	
				if (v[1] == 0.){	// Cutoff Region
		
					isd1L3 = 0.;
			
				}else if (v[1] <= Vdsat){	// Linear Region
		
				   DeltaL = sqrt(dparm[KAPA]*XD*XD*Vdsat/8.);
				   DeltaL *= (v[1]/Vdsat)*(v[1]/Vdsat)*(v[1]/Vdsat);

					if (DeltaL > 0.5*Leff){

						DeltaL = Leff-Leff*Leff/(4*DeltaL);
						isd1L3 = B*(v[0]-Vthp-(1.+FB)*v[1]/2.)*v[1];
						isd1L3 /= 1.+dparm[THET]*(v[0]-Vthp);
						isd1L3 /= 1.+Us*v[1]/(Leff*dparm[VMAX]);
						isd1L3 *= Leff/(Leff-DeltaL);

					}else{

						isd1L3 = B*(v[0]-Vthp-(1.+FB)*v[1]/2.)*v[1];			
						isd1L3 /= 1.+dparm[THET]*(v[0]-Vthp);
						isd1L3 /= 1.+Us*v[1]/(Leff*dparm[VMAX]);
						isd1L3 *= Leff/(Leff-DeltaL);		
					}
			
				}else{		// Saturation Region

					DeltaL = sqrt(dparm[KAPA]*XD*XD*(v[1]-Vdsat+Vdsat/8.));

					if (DeltaL > 0.5*Leff) {
			
						DeltaL = Leff-Leff*Leff/(4*DeltaL);
						isd1L3 = B*(v[0]-Vthp-(1.+FB)*Vdsat/2.)*Vdsat;
						isd1L3 /= (1.+dparm[THET]*(v[0]-Vthp));
						isd1L3 /= 1.+Us*Vdsat/(Leff*dparm[VMAX]);
						isd1L3 *= Leff/(Leff-DeltaL);
				
					}else{
			
						isd1L3 = B*(v[0]-Vthp-(1.+FB)*Vdsat/2.)*Vdsat;
						isd1L3 /= (1.+dparm[THET]*(v[0]-Vthp));
						isd1L3 /= 1.+Us*Vdsat/(Leff*dparm[VMAX]);
						isd1L3 *= Leff/(Leff-DeltaL);			

					}
			
				}
    
    			}
	
		}else{    // VMAX >0
    
			if (v[0] <= Von){	// Weak Inversion							

				if (v[1] == 0.){		// Cutoff Region
        
					isd1L3 = 0.;
	

				}else if (v[1] <= Vdsat){	// Linear Region  
		
		
					isd1L3 = safe_exp((v[0]-Von)/(n*tvar[VT]));
					isd1L3 *= B*(Von-Vthp-(1.+FB)*v[1]/2.)*v[1];
					isd1L3 /= (1.+dparm[THET]*(Von-Vthp));	
					isd1L3 /= (1.+Us*v[1]/(Leff*dparm[VMAX]));		
		
		
				}else{		// Saturation Region
				
					currDouble A = 1.+Us*Vdsat/(Leff*dparm[VMAX]);
					currDouble A1 = (Von-Vthp-(1.+FB)*Vdsat)/A;
					currDouble A2 = (Von-Vthp-(1.+FB)*Vdsat/2.)*Vdsat*Us/(Leff*dparm[VMAX]);
			           		   A2 /= (A*A);
					currDouble gd = B*(A1-A2)/(1.+dparm[THET]*(Von-Vthp));
					currDouble A3 = B*(Von-Vthp-(1.+FB)*Vdsat/2.)*Vdsat;
				   		   A3 /= A*(1.+dparm[THET]*(Von-Vthp));
					currDouble Ep = dparm[KAPA]*A3/(gd*Leff);
					currDouble X = Ep*XD*XD/2.;
					DeltaL = sqrt(X*X+dparm[KAPA]*XD*XD*(v[1]-Vdsat))-X;
			
		
					if (DeltaL > 0.5*Leff) {	//punch through approximation
			
						DeltaL = Leff-Leff*Leff/(4*DeltaL);
						isd1L3 = safe_exp((v[0]-Von)/(n*tvar[VT]));
						isd1L3 *= B*(Von-Vthp-(1.+FB)*Vdsat/2.)*Vdsat;
						isd1L3 /= A*(1.+dparm[THET]*(Von-Vthp));
						isd1L3 *= Leff/(Leff-DeltaL);
						
				
					}else{
			
						isd1L3 = safe_exp((v[0]-Von)/(n*tvar[VT]));
						isd1L3 *= B*(Von-Vthp-(1.+FB)*Vdsat/2.)*Vdsat;
						isd1L3 /= A*(1.+dparm[THET]*(Von-Vthp));		
						isd1L3 *= Leff/(Leff-DeltaL);
				
					}	
			
				}
    		
    			}else{		// Strong Inversion
	
				if (v[1] == 0.){	// Cutoff Region
		
					isd1L3 = 0.;
			
			
				}else if (v[1] <= Vdsat){	// Linear Region  
		
		
					isd1L3 = B*(v[0]-Vthp-(1.+FB)*v[1]/2.)*v[1];
					isd1L3 /= 1.+dparm[THET]*(v[0]-Vthp);			
					isd1L3 /= 1.+Us*v[1]/(Leff*dparm[VMAX]);
					
			
				}else{		// Saturation Region
			
					currDouble A = 1.+Us*Vdsat/(Leff*dparm[VMAX]);
					currDouble A1 = (v[0]-Vthp-(1.+FB)*Vdsat)/A;
					currDouble A2 = (v[0]-Vthp-(1.+FB)*Vdsat/2.)*Vdsat*Us/(Leff*dparm[VMAX]);
			           		   A2 /= (A*A);
					currDouble gd = B*(A1-A2)/(1.+dparm[THET]*(v[0]-Vthp));
					currDouble A3 = B*(v[0]-Vthp-(1.+FB)*Vdsat/2.)*Vdsat;
				   		   A3 /= A*(1.+dparm[THET]*(v[0]-Vthp));
					currDouble Ep = dparm[KAPA]*A3/(gd*Leff);
					currDouble X = Ep*XD*XD/2.;
					DeltaL = sqrt(X*X+dparm[KAPA]*XD*XD*(v[1]-Vdsat))-X;
			
		
					if (DeltaL > 0.5*Leff) {	//punch through approximation
			
						DeltaL = Leff-Leff*Leff/(4*DeltaL);
						isd1L3 = B*(v[0]-Vthp-(1.+FB)*Vdsat/2.)*Vdsat;
						isd1L3 /= A*(1.+dparm[THET]*(v[0]-Vthp));
						isd1L3 *= Leff/(Leff-DeltaL);
				
				
					}else{
			
						isd1L3 = B*(v[0]-Vthp-(1.+FB)*Vdsat/2.)*Vdsat;
						isd1L3 /= A*(1.+dparm[THET]*(v[0]-v[1]-Vthp));
						isd1L3 *= Leff/(Leff-DeltaL);
				
					}
			
				}
			
			}
    
    		}

	}else{		// Reverse mode
	
		if (dparm[VMAX] <= 0.){		//Vmax <= 0
    
    			if (v[0]-v[1] <= Von){	// Weak Inversion

				if (-v[1] == 0.){	// Cutoff Region
        
					isd1L3 = 0.;
	
				}else if (-v[1] <= Vdsat){	// Linear Region
				
					DeltaL = sqrt(dparm[KAPA]*XD*XD*Vdsat/8.);
				   	DeltaL *= (-v[1]/Vdsat)*(-v[1]/Vdsat)*(-v[1]/Vdsat);

					if (DeltaL > 0.5*Leff){

						DeltaL = Leff-Leff*Leff/(4*DeltaL);
						isd1L3 = safe_exp((v[0]-v[1]-Von)/(n*tvar[VT]));
						isd1L3 *= -B*(Von-Vthp-(1.+FB)*(-v[1])/2.)*(-v[1]);
						isd1L3 /= (1.+dparm[THET]*(Von-Vthp));
						isd1L3 /= 1.+Us*(-v[1])/(Leff*dparm[VMAX]);
						isd1L3 *= Leff/(Leff-DeltaL);
					

					}else{

						isd1L3 = safe_exp((v[0]-v[1]-Von)/(n*tvar[VT]));
						isd1L3 *= -B*(Von-Vthp-(1.+FB)*(-v[1])/2.)*(-v[1]);
						isd1L3 /= (1.+dparm[THET]*(Von-Vthp));
						isd1L3 /= 1.+Us*(-v[1])/(Leff*dparm[VMAX]);
						isd1L3 *= Leff/(Leff-DeltaL);				
								
					}
				
				
				}else{		// Saturation region
				
					DeltaL = sqrt(dparm[KAPA]*XD*XD*(-v[1]-Vdsat+Vdsat/8.));

					if (DeltaL > 0.5*Leff) {
			
						DeltaL = Leff-Leff*Leff/(4*DeltaL);
						isd1L3 = safe_exp((v[0]-v[1]-Von)/(n*tvar[VT]));
						isd1L3 *= -B*(Von-Vthp-(1.+FB)*Vdsat/2.)*Vdsat;
						isd1L3 /= 1.+dparm[THET]*(Von-Vthp);
						isd1L3 /= 1.+Us*Vdsat/(Leff*dparm[VMAX]);
						isd1L3 *= Leff/(Leff-DeltaL);
										
					}else{
						
						isd1L3 = safe_exp((v[0]-v[1]-Von)/(n*tvar[VT]));
						isd1L3 *= -B*(Von-Vthp-(1.+FB)*Vdsat/2.)*Vdsat;
						isd1L3 /= 1.+dparm[THET]*(Von-Vthp);
						isd1L3 /= 1.+Us*Vdsat/(Leff*dparm[VMAX]);
						isd1L3 *= Leff/(Leff-DeltaL);

					}
				
				}
    
    			}else{		// Strong Inversion
	
				if (-v[1] == 0.){	// Cutoff Region
		
					isd1L3 = 0.;
			
				}else if (-v[1] <= Vdsat){	// Linear Region
		
					DeltaL = sqrt(dparm[KAPA]*XD*XD*Vdsat/8.);
				   	DeltaL *= (-v[1]/Vdsat)*(-v[1]/Vdsat)*(-v[1]/Vdsat);

					if (DeltaL > 0.5*Leff){

						DeltaL = Leff-Leff*Leff/(4*DeltaL);
						isd1L3 = -B*(v[0]-v[1]-Vthp-(1.+FB)*(-v[1])/2.)*(-v[1]);
						isd1L3 /= 1.+dparm[THET]*(v[0]-v[1]-Vthp);
						isd1L3 /= 1.+Us*(-v[1])/(Leff*dparm[VMAX]);
						isd1L3 *= Leff/(Leff-DeltaL);

					}else{

						isd1L3 = -B*(v[0]-v[1]-Vthp-(1.+FB)*(-v[1])/2.)*(-v[1]);
						isd1L3 /= 1.+dparm[THET]*(v[0]-v[1]-Vthp);
						isd1L3 /= 1.+Us*(-v[1])/(Leff*dparm[VMAX]);
						isd1L3 *= Leff/(Leff-DeltaL);		
					}
			
				}else{		// Saturation Region

					DeltaL = sqrt(dparm[KAPA]*XD*XD*(-v[1]-Vdsat+Vdsat/8.));

					if (DeltaL > 0.5*Leff) {
			
						DeltaL = Leff-Leff*Leff/(4*DeltaL);
						isd1L3 = -B*(v[0]-v[1]-Vthp-(1.+FB)*Vdsat/2.)*Vdsat;
						isd1L3 /= (1.+dparm[THET]*(v[0]-v[1]-Vthp));
						isd1L3 /= 1.+Us*Vdsat/(Leff*dparm[VMAX]);
						isd1L3 *= Leff/(Leff-DeltaL);
				
					}else{
			
						isd1L3 = -B*(v[0]-v[1]-Vthp-(1.+FB)*Vdsat/2.)*Vdsat;
						isd1L3 /= (1.+dparm[THET]*(v[0]-v[1]-Vthp));
						isd1L3 /= 1.+Us*Vdsat/(Leff*dparm[VMAX]);
						isd1L3 *= Leff/(Leff-DeltaL);			

					}
			
				}
    
    			}
	
		}else{    // VMAX >0
    
			if (v[0]-v[1] <= Von){	// Weak Inversion							

				if (-v[1] == 0.){		// Cutoff Region
        
					isd1L3 = 0.;
	
				}else if (-v[1] <= Vdsat){	// Linear Region  
		
					isd1L3 = safe_exp((v[0]-v[1]-Von)/(n*tvar[VT]));
					isd1L3 *= -B*(Von-Vthp-(1.+FB)*(-v[1])/2.)*(-v[1]);
					isd1L3 /= 1.+dparm[THET]*(Von-Vthp);
					isd1L3 /= 1.+Us*(-v[1])/(Leff*dparm[VMAX]);			
		
				}else{		// Saturation Region
						
						
					currDouble A = 1.+Us*Vdsat/(Leff*dparm[VMAX]);
					currDouble A1 = (Von-Vthp-(1.+FB)*Vdsat)/A;
					currDouble A2 = (Von-Vthp-(1.+FB)*Vdsat/2.)*Vdsat*Us/(Leff*dparm[VMAX]);
			           		   A2 /= (A*A);
					currDouble gd = B*(A1-A2)/(1.+dparm[THET]*(Von-Vthp));
					currDouble A3 = B*(Von-Vthp-(1.+FB)*Vdsat/2.)*Vdsat;
				   		   A3 /= A*(1.+dparm[THET]*(Von-Vthp));
					currDouble Ep = dparm[KAPA]*A3/(gd*Leff);
					currDouble X = Ep*XD*XD/2.;
					DeltaL = sqrt(X*X+dparm[KAPA]*XD*XD*(-v[1]-Vdsat))-X;
			
		
					if (DeltaL > 0.5*Leff) {	//punch through approximation
			
						DeltaL = Leff-Leff*Leff/(4*DeltaL);
						isd1L3 = safe_exp((v[0]-v[1]-Von)/(n*tvar[VT]));
						isd1L3 *= -B*(Von-Vthp-(1.+FB)*Vdsat/2.)*Vdsat;
						isd1L3 /= 1.+dparm[THET]*(Von-Vthp);
						isd1L3 /= 1.+Us*Vdsat/(Leff*dparm[VMAX]);	
						isd1L3 *= Leff/(Leff-DeltaL);
						
					}else{
			
						isd1L3 = safe_exp((v[0]-v[1]-Von)/(n*tvar[VT]));
						isd1L3 *= -B*(Von-Vthp-(1.+FB)*Vdsat/2.)*Vdsat;
						isd1L3 /= 1.+dparm[THET]*(Von-Vthp);
						isd1L3 /= 1.+Us*Vdsat/(Leff*dparm[VMAX]);	
						isd1L3 *= Leff/(Leff-DeltaL);

					}	

				}
    		
    			}else{		// Strong Inversion
	
				if (-v[1] == 0.){	// Cutoff Region
		
					isd1L3 = 0.;
			
				}else if (-v[1] <= Vdsat){	// Linear Region  
		
		
					isd1L3 = -B*(v[0]-v[1]-Vthp-(1.+FB)*(-v[1])/2.)*(-v[1]);
					isd1L3 /= 1.+dparm[THET]*(v[0]-v[1]-Vthp);
					isd1L3 /= 1.+Us*(-v[1])/(Leff*dparm[VMAX]);			
			
			
				}else{		// Saturation Region
			
					currDouble A = 1.+Us*Vdsat/(Leff*dparm[VMAX]);
					currDouble A1 = (v[0]-v[1]-Vthp-(1.+FB)*Vdsat)/A;
					currDouble A2 = (v[0]-v[1]-Vthp-(1.+FB)*Vdsat/2.)*Vdsat*Us/(Leff*dparm[VMAX]);
			           		   A2 /= (A*A);
					currDouble gd = B*(A1-A2)/(1.+dparm[THET]*(v[0]-v[1]-Vthp));
					currDouble A3 = B*(v[0]-v[1]-Vthp-(1.+FB)*Vdsat/2.)*Vdsat;
				   		   A3 /= A*(1.+dparm[THET]*(v[0]-v[1]-Vthp));
					currDouble Ep = dparm[KAPA]*A3/(gd*Leff);
					currDouble X = Ep*XD*XD/2.;
					DeltaL = sqrt(X*X+dparm[KAPA]*XD*XD*(-v[1]-Vdsat))-X;
			
		
					if (DeltaL > 0.5*Leff) {	//punch through approximation
			
						DeltaL = Leff-Leff*Leff/(4*DeltaL);
						isd1L3 = -B*(v[0]-v[1]-Vthp-(1.+FB)*Vdsat/2.)*Vdsat;
						isd1L3 /= A*(1.+dparm[THET]*(v[0]-v[1]-Vthp));
						isd1L3 *= Leff/(Leff-DeltaL);
				
				
					}else{
			
						isd1L3 = -B*(v[0]-v[1]-Vthp-(1.+FB)*Vdsat/2.)*Vdsat;
						isd1L3 /= A*(1.+dparm[THET]*(v[0]-v[1]-Vthp));
						isd1L3 *= Leff/(Leff-DeltaL);
				
					}
				}
			}
    		}
	}
}
//*********************************************************************************** 
      
      // Note: time delay not implemented for now
      // Calculate ids with few temporaries (we should do this
      // everywhere)
  

   // Adjust variables to a new temperature.
  
  template <typename tvarDouble, typename varDouble, typename tempDouble>
  inline void setTemp(tvarDouble* tvar, 
		      varDouble* dparm, varDouble* cvar, tempDouble& T)

  
  { // Parameter calculation 
   
   tempDouble Tabs = T + tzero; 
   tvar[VT]  = kBoltzman*Tabs/eCharge;
   tvar[PHI] = 2.*tvar[VT]*log(dparm[NSUBP]/dparm[NI]); 
   tvar[EGFET]=1.16-(7.02e-4*Tabs*Tabs)/(Tabs+1108.);

  }
  
   //For efficiency, we may want to pre-calculate some expressions
  // that do not change with voltage or temperature.  Adjust variables
  // to a new parameter set.
  template <typename varDouble>
  inline void newParms(varDouble* cvar, varDouble* dparm)
  {
  
  }

  // Calculate power. pout initially contains the current.
  //template <typename powerDouble, typename voltDouble, 
	//    typename paramDouble>
  //inline void power(powerDouble& pout, voltDouble* v, 
		//    paramDouble* dparm)
//  {
  //  pout *= v[1];
  //}

  //private:
  
  // Pointer to integer parameter vector from macroelement
  //  int *iparm; // not necessary for this model

};

// No thermal port
typedef FADBADGenericVCS<Isd1L3_eval, GenericVCCS, 3, 
			 MAX_DPARM_INDEX, TEMP, false> Isd1L3;
			 

			 
			 
// With thermal port
//typedef FADBADTGenericVCS<IdsCurtice_eval, GenericVCCS, 3, 
			//  MAX_DPARM_INDEX, TEMP> TIdsCurtice;
// Power source
//typedef FADBADPowerSource<IdsCurtice_eval, 3, 
 		//	  MAX_DPARM_INDEX, TEMP> IdsPS;

#endif
