// This may look like C code, but it is really -*- C++ -*-
//
// PMOS Drain-Gate Capacitance implementation using Yang-Chatterjee charge model
// between gate and drain terminal  
//
//          
// 

#ifndef Qdp_H
#define Qdp_H 1

class Qdp_eval {
public:
  // This are locally-defined variables
  enum {VT, PHI, MAX_TVAR_INDEX};
  enum {dummy, MAX_CVAR_INDEX};

  Qdp_eval(int *iparm) {} // iparm not needed here

  // Main evaluation function. Only voltage dependence considered here.
  template <typename currDouble, typename voltDouble, typename paramDouble, 
	    typename varDouble, typename cvarDouble>
  inline void operator()(currDouble& qdp, voltDouble* v, 
			 paramDouble* dparm, 
			 varDouble* tvar, cvarDouble* cvar)
  {

	//v[0]=Vsg v[1]=Vsd v[2]=Vsb	
	  
    currDouble Vthp;
    currDouble phibs;
    currDouble sqphbs;
    
    paramDouble Cox = dparm[EOX]/dparm[TOX];
    paramDouble Leff = dparm[L]-2.*dparm[LD];
    paramDouble Co = Cox*dparm[W]*Leff;
    paramDouble Gama = sqrt(2.*dparm[ES]*eCharge*dparm[NSUBP])/Cox;
    paramDouble Sigma = dparm[ETA]*8.15e-22/(Cox*Leff*Leff*Leff);
    paramDouble XD = sqrt(2.*dparm[ES]/(eCharge*dparm[NSUBP]));
    paramDouble FN = dparm[ES]*dparm[DELT]*3.141592654/(4.*Cox*dparm[W]);

        if (v[1] >= 0. && v[2] <= 0.) {	
  	
    	phibs = tvar[PHI]-v[2];
    	sqphbs = sqrt(phibs);
    
    	}else if (v[1] < 0. && v[2]-v[1] <= 0.){
    	
    	phibs = tvar[PHI]-(v[2]-v[1]);
    	sqphbs = sqrt(phibs);
    		   
        }else if (v[1] >= 0. && v[2] > 0.){
        
        //sqphbs = sqrt(tvar[PHI])/(1+0.5*(v[2])/tvar[PHI]+0.375*(v[2])*(v[2])/(tvar[PHI]*tvar[PHI]));
        sqphbs = sqrt(tvar[PHI])/(1.+0.5*v[2]/tvar[PHI]);
        phibs = sqphbs*sqphbs;
        	   
        }else{
        
        //sqphbs = sqrt(tvar[PHI])/(1+0.5*(v[2]-v[1])/tvar[PHI]+0.375*(v[2]-v[1])*(v[2]-v[1])/(tvar[PHI]*tvar[PHI]));
        sqphbs = sqrt(tvar[PHI])/(1.+0.5*(v[2]-v[1])/tvar[PHI]);
        phibs = sqphbs*sqphbs;
        
        }
            
    if (dparm[VTC] ==1){
    
    	Vthp = dparm[VT0P]*(-1.)+Gama*(sqphbs-sqrt(tvar[PHI]));
    	
    }else{
    
    	currDouble WP = XD*sqphbs;
    	currDouble WcovXJ = 0.0631353+0.8013292*(WP/dparm[XJ])-0.011107*(WP/dparm[XJ])*(WP/dparm[XJ]);
    	currDouble FS = sqrt(1.-(WP/(dparm[XJ]+WP))*(WP/(dparm[XJ]+WP)));
    	       	   FS *= (dparm[LD]/dparm[XJ]+WcovXJ);
	           FS -= dparm[LD]/dparm[XJ];
    	           FS *= (-1.)*(dparm[XJ]/Leff);
   	           FS += 1.;
   	       
    	currDouble Vbi = (dparm[VT0P]+Gama*sqrt(tvar[PHI]))*(-1.);
    
    	if (v[1] >= 0){
	
    		Vthp = Vbi-Sigma*(v[1])+Gama*FS*sqphbs+FN*phibs;
    		
    	}else{
    	
    		Vthp = Vbi-Sigma*(-v[1])+Gama*FS*sqphbs+FN*phibs;
    	
    	}
    
    }    
    
    	qdp = dparm[W]*dparm[CGDO]*(-v[1]+v[0]);
    			
    
	if (v[1] >= 0.){
	
	
    
		if (v[0] > v[1]+Vthp ) {							
    			
			qdp += Co*(v[1]*v[1]/(8.*(v[0]-Vthp-v[1]/2.))+(v[0]-Vthp)/2.-3./4.*v[1]);
			
			
	  
		}
     	}else{
     
     		if (Vthp < v[0]-v[1] && v[0]-v[1] <= -v[1]+Vthp) {	// Saturation region
	
			qdp += 2./3.*Co*(v[0]-v[1]-Vthp);
			
	
		}else if (v[0]-v[1] > -v[1]+Vthp){			// Linear region
	  
			qdp += Co*((v[0]-v[1]-Vthp)/2.+1./4.*(-v[1])-(-v[1]*-v[1])/(24*(v[0]-v[1]-Vthp-(-v[1])/2)));
				
			
		}
	} 
}

  // Adjust variables to a new temperature.
  template <typename tvarDouble, typename varDouble, typename tempDouble>
  inline void setTemp(tvarDouble* tvar, 
		      varDouble* dparm, varDouble* cvar, tempDouble& T)
  {
  
   tempDouble Tabs = T + tzero; 
   tvar[VT]  = kBoltzman*Tabs/eCharge;
   tvar[PHI] = 2.*tvar[VT]*log(dparm[NSUBP]/dparm[NI]); 
   
   
  
  }
  
  // For efficiency, we may want to pre-calculate some expressions
  // that do not change with voltage nor temperature.  Adjust variables
  // to a new parameer set.
  template <typename varDouble>
  inline void newParms(varDouble* cvar, varDouble* dparm)
  {
  
  
  //cvar[Cgdo_1] = cvar[Cox_1]*dparm[OVERLAP];
  }

  //private:
  
  // Pointer to integer parameter vector from macroelement
  //  int *iparm; // not necessary for this model

};


typedef FADBADGenericVCS<Qdp_eval, GenericVCQS, 3, 
			 MAX_DPARM_INDEX, TEMP, false> Qdp;
// With thermal port
//typedef FADBADTGenericVCS<DiodeCharge_eval, GenericVCQS, 2, 
//			  MAX_DPARM_INDEX, TEMP> TDiodeCharge;

#endif
