// This may look like C code, but it is really -*- C++ -*-
//
// Implements a regular resistor with temperature port (nonlinear)
//
// 

#ifndef NLRType_H
#define NLRType_H 1

class NLRType_eval {
public:

  enum {RES, MAX_CVAR_INDEX};
  enum {DUMMY, MAX_TVAR_INDEX};

  NLRType_eval(int *iparm) {} // iparm not needed here

  // Main evaluation function. Voltage and temperature dependence
  // considered here (since otherwise resistor is linear).
  template <typename currDouble, typename voltDouble, typename paramDouble, 
	    typename varDouble, typename cvarDouble>
  inline void operator()(currDouble& i, voltDouble* v, 
			 paramDouble* dparm, 
			 varDouble* tvar, cvarDouble* cvar)
  {
    // v[0]: voltage across resistor
    // v[1]: temperature
    currDouble dt = v[1] - dparm[TNOM];
    i = v[0] / cvar[RES] / (1. + (dparm[TC1] + dparm[TC2] * dt) * dt);
  }

  // Adjust variables to a new temperature.
  template <typename tvarDouble, typename varDouble, typename tempDouble>
  inline void setTemp(tvarDouble* tvar, 
		      varDouble* dparm, varDouble* cvar, tempDouble& T)
  {
    // Do nothing
  }
  
  // For efficiency, we may want to pre-calculate some expressions
  // that do not change with voltage or temperature.  Adjust variables
  // to a new parameer set.
  template <typename varDouble>
  inline void newParms(varDouble* cvar, varDouble* dparm)
  {
    // Calculate resistance. Assume all parameters are checked
    if (dparm[RSH] == 0.) 
      cvar[RES] = dparm[R];
    else
      cvar[RES] = dparm[RSH] * (dparm[L]-dparm[NARROW]) 
	/ (dparm[W]-dparm[NARROW]);
  }

  // Calculate power. pout initially contains the current.
  template <typename powerDouble, typename voltDouble, 
	    typename paramDouble>
  inline void power(powerDouble& pout, voltDouble* v, 
		    paramDouble* dparm)
  {
    pout *= v[0];
  }


};

// With thermal port (already included in control voltages)
typedef FADBADGenericVCS<NLRType_eval, GenericVCCS, 2, 
			 MAX_DPARM_INDEX, TEMP, false> TRType;
// Power source
typedef FADBADPowerSource<NLRType_eval, 2, 
 			  MAX_DPARM_INDEX, TEMP> RTypePS;

#endif
