// This may look like C code, but it is really -*- C++ -*-
//
// 
//
// by Carlos E. Christoffersen

#ifndef ThermalVCCS_h
#define ThermalVCCS_h 1

#include <carrot/LinearfVCCS.h>

typedef std::complex<double> double_complex;

class ThermalVCCS : public LinearfVCCS
{
public:

  ThermalVCCS(const int& nfreq, double* freq, double_complex* Rth) 
    : LinearfVCCS(), nfreq(nfreq), freq(freq), Rth(Rth) {}

  virtual ~ThermalVCCS() {}
    
  // This returns the complex gain of the VCCS. 
  virtual void getGain(double_complex& y, const double& f)
  {
    // Search f in frequency vector
    int index(0);
    while (index < nfreq) {
      if (freq[index] > f)
	break;
      index++;
    }
    // Now freq[index] > f and freq[index-1] < f 
    // or
    // f is greater than the maximum frequency we have. It should be
    // safe to return zero in this case.
    if (index == nfreq) {
      y = 0.;
    } else {
      // Just use linear interpolation
      y = Rth[index-1] + (Rth[index]-Rth[index-1]) 
	* (f-freq[index-1]) / (freq[index] - freq[index-1]);
      const double &scale = options.getDHandler()->getParmVec()[OPT_SCALE];
      y *= scale * scale;
    }
  }

private:
  
  int nfreq;
  double* freq;
  double_complex* Rth;

};

#endif


