#include "VCCS.h"

static ItemInfo VCCS_info = {
  "vccs",
  "Voltage-controlled current source",
  "Carlos Christoffersen"
  "DEFAULT_ADDRESS"
};

// Double Parameters
// Make sure enum and ParInfo definitions are in the same order!
enum {GAIN, MAX_DPARM_INDEX};

//Insert ParmInfo values here
static ParamHandler<double>::ParmInfo VCCS_dparams[] = { 
  {"g", "Gain (S)", true, 0.}
};

// Uncomment for integer parameters
//enum {MAX_IPARM_INDEX};
//static ParamHandler<int>::ParmInfo VCCS_iparams[] = {  };


//Library loading mechanism
extern "C" {
  const char * ModuleName(){
    return VCCS_info.type.c_str();
  }
  ParmSet * newParmSet(const char * model){
    return new ParmSet(model,
		         &VCCS_info,
		         VCCS_dparams, MAX_DPARM_INDEX); 
//		         VCCS_iparams, MAX_IPARM_INDEX);
  }
  void delParmSet(ParmSet * p){
    delete p;
    p=0;
  }
  MacroElement * newObject(ParmSet * p, const char * name){
    return new VCCS(p, name);
  }
  void delObject(MacroElement * e){
    delete e;
    e = 0;
  }
};

#include "GType.h"

VCCS::VCCS(ParmSet *pset, const std::string& name)
  :MacroElement(pset, name, 4)
{
  //Constructor code goes here (above, 4 is the number of terminals)
}

VCCS::~VCCS()
{
  //Destructor code goes here
}

void VCCS::init()
{
  // access to double parameters
  double *dparm = getParmSet()->getDHandler()->getParmVec();
  // Check required parameters
  getParmSet()->checkParms();
  // Check external terminal connections
  assert(checkTerms());
  // Set number of internal nodes
  setNInternalNodes(0);

  LinearVCCS* lvccs = new GType(dparm, NULL);
  static const int tvec[4] = {0,1,2,3};
  insert(lvccs, tvec);
}


