#ifndef GLOBAL_H
#define GLOBAL_H

#include <math.h>

// Global constants, etc.

const double pi = 3.14159265358979323846;
const double twopi = pi * 2.;
const double deg2rad = pi / 180.;

// Boltzman constant
const double kBoltzman = 1.3806266e-23;
// Electron charge
const double eCharge = 1.6021918e-19;
// Permittivity of free space 
const double epsilon0 = 8.854194e-12;
// Permeability of free space
const double mu0 = pi * 4e-7;
// Speed of light in free space
const double c0 = 2.998633e+08;
// Absolute temperature of 0 degree Celsius
const double tzero = 273.15;

// Use this instead of exp() to avoid overflows (th may need some
// adjustement).
template<typename mydouble>
mydouble safe_exp(mydouble x)
{
  const double th(50);
  if (x < th)
    return exp(x);
  else
    return exp(th) * (x - th + 1.);
}

// Use this to overcome lack of implementation of tanh in FADBAD++
template<typename mydouble>
mydouble my_tanh(mydouble x)
{
  const double th(50);
  mydouble y;
  if (x < th && x > -th) {
    mydouble ex = exp(x);
    mydouble emx = exp(-x);
    y = ex - emx;
    ex += emx;
    y /= ex;
  } else if (x < -50)
    y = -1.;
  else 
    y = 1.;
  return y;
}

#endif
