// This may look like C code, but it is really -*- C++ -*-
//
// This class is to hold DC values, transient voltages for all saved
// nodes. It should be as light as possible since one of these is
// required for each sweep/time sample.
//
// by Carlos E. Christoffersen

#ifndef DoubleHolder_h
#define DoubleHolder_h 1

#include <vector>
#include <string>
#include <cassert>
#include <cstdio>

class DoubleHolder : public SampleHolder
{
public:

  DoubleHolder(const int& vecsize) 
  {
    assert(vecsize > 0);
    uvec = new double [vecsize];
  }
  
  virtual ~DoubleHolder() 
  {
    delete [] uvec;
  }

  // Write node(index) information to out string
  virtual void writeNode(const int& index, std::string& out)
  {
    char buffer[256];
    sprintf(buffer,"%g\t %g", this->getVal(), uvec[index]);
    out = buffer;
  }

  // Set values
  inline void saveValue(const int& index, const double& val)
  {
    uvec[index] = val;
  }

private:
  
  double* uvec;

};


#endif


