// This may look like C code, but it is really -*- C++ -*-
//
// This is a base class for element nodes, which are stored in a
// graph. The order of the terminals in termVec *is important*. 
//
// by Carlos E. Christoffersen

#ifndef ElementNode_h
#define ElementNode_h 1

#include <carrot/CircuitItem.h>
#include <vector>

// Defined in Terminal.h
class Terminal;

class ElementNode : public CircuitItem
{
public:

  virtual ~ElementNode();

  // Link terminal (add terminal to list)
  // Use newterm = NULL to unlink terminals by number.
  void link(const unsigned& index, Terminal* newterm);

  // Unlink by pointer. Terminals need this. If terminal connected
  // multiple times, all connections are removed.
  void unlink(Terminal* oldterm);
  
  // Get terminal by index
  inline Terminal* getTerminal(const unsigned& index)
  {
    assert(index < termVec.size());
    return termVec[index];
  }

  inline unsigned getNTerms()
  {
    return termVec.size();
  }

  bool checkTerms();
  
protected:
  
  ElementNode(const unsigned& nterms);

  // Use with care. Do not set nterms to a number smaller than the
  // previous number of terminals if already connected.
  void setNTerms(const unsigned& nterms);
  
private:

  // List of nodes connected
  std::vector<Terminal*> termVec;
};

typedef std::vector<ElementNode*> ElemVector;

#endif


