// This may look like C code, but it is really -*- C++ -*-
//
// Implements a LinearVCCS/QS using automatic differentiation
//
//                
// 

#ifndef FADBADLinearVCS_H
#define FADBADLinearVCS_H

#include <carrot/LinearVCQS.h>
// #include <fadbad/badiff.h>
#include <fadbad/fadiff.h>

// EV: evaluation class
// ndparms: number of parameters (type double)

template <typename EV, typename LVCS, unsigned ndparms>
class FADBADLinearVCS : public LVCS {
public:

  // dparm and iparm values are maintained by the MacroElement
  // (here we assume these are ready to use)
  FADBADLinearVCS(double* dparm, int* iparm)
    : LVCS(0.), dparm(dparm), teval(iparm) 
  {
    // Calculate gain according to parameters and store result
    double g;
    teval(g, dparm);
    this->setGain(g);
  }

  virtual ~FADBADLinearVCS() {}

  // Call this each time the parameter values are changed
  void init()
  {
    // Calculate gain according to parameters and store result
    double g;
    teval(g, dparm);
    this->setGain(g);
  }

  virtual bool paramDeriv(double* dg);

private:

  // Parameter definitions
  double *dparm;
  EV teval;

};

template <typename EV, typename LVCS, unsigned ndparms>
bool FADBADLinearVCS<EV, LVCS, ndparms>::
paramDeriv(double* dg)
{
  F<double> fdparm[ndparms], fg;
  for (unsigned i(0); i < ndparms; i++) {
    fdparm[i] = dparm[i];
    fdparm[i].diff(i, ndparms); // differenciate respect to one parameter
  }
  // Calculate gain
  teval(fg, fdparm);
  // Get derivatives
  for (unsigned i(0); i < ndparms; i++) {
    dg[i] = fg.d(i);
  }
  return true;
}


#endif
