// This may look like C code, but it is really -*- C++ -*-
//
// Implements a single-time IndependentCS using automatic differentiation
//
//                
// 

#ifndef FADBADMIndependentCS_H
#define FADBADMIndependentCS_H

// #include <fadbad/badiff.h>
#include <fadbad/fadiff.h>

// EV: evaluation class
// ndparms: number of parameters (type double)
template <typename EV, unsigned ndparms>
class FADBADMIndependentCS : public IndependentCS {
public:

  // dparm and iparm values are maintained by the MacroElement
  // (here we assume these are ready to use)
  FADBADMIndependentCS(double* dparm, int* iparm)
    : IndependentCS(), dparm(dparm), teval(iparm) {}

  virtual ~FADBADMIndependentCS() {}

  // Call this each time the parameter values are changed
  // (no need for virtual here?)
  void init()
  {
    // Check that terminals are connected.  Only needed for
    // debugging. Properly written MacroElements should take care of
    // this.
    assert(checkTerms());
    // Adjust teval variables to new parameters.
    teval.newParms(cvar, dparm);
  }

  // These are defined in the base classes
  virtual void eval(double& j, 
		    const double& ctime1,
		    double ctime2 = -1.,
		    double ctime3 = -1.)
  {
    if (ctime2 == -1.)
      // Time-shift second time variable according to deltaT2 (default:0)
      ctime2 = ctime1 - getDeltaT2();
    teval(j, ctime1, ctime2, dparm, cvar);
  }

  virtual bool paramDeriv(double* df, 
			  const double& ctime1,
			  double ctime2 = -1.,
			  double ctime3 = -1.)
  {
    if (ctime2 == -1.)
      ctime2 = ctime1;
    F<double> fdparm[ndparms], fcvar[EV::MAX_CVAR_INDEX], fi;
    for (unsigned i(0); i < ndparms; i++) {
      fdparm[i] = dparm[i];
      fdparm[i].diff(i, ndparms); // differenciate respect to one parameter
    }
    // Consider all variables 
    teval.newParms(fcvar, fdparm);
    // evaluate function
    teval(fi, ctime1, ctime2, fdparm, fcvar);   
    for (unsigned i(0); i < ndparms; i++) {
      df[i] = fi.d(i);
    }
    return true;
  }

private:
  
  // Parameter definitions
  double *dparm;
  double cvar[EV::MAX_CVAR_INDEX];
  EV teval;
  
};


#endif
