// This may look like C code, but it is really -*- C++ -*-
//
// Implements a thermal power source. 
//
//          1                2      4
//          o                o +    o +           
//          |                           
//         / \                                            ---
//        / ^ \                V      I
//        \ | / P = V * I      
//         \ /
//          |                          
//          o     	     o -    o -   
//          0                3      5
// 

#ifndef ThermalPowerS_H
#define ThermalPowerS_H 1

#include <carrot/Options.h>

class ThermalPowerS_eval {
public:

  enum {DUMMY1, MAX_TVAR_INDEX};
  enum {DUMMY2, MAX_CVAR_INDEX};

  ThermalPowerS_eval(int *iparm) {} // iparm not needed here

  // Main evaluation function. Only voltage dependence considered here.
  template <typename currDouble, typename voltDouble, typename paramDouble, 
	    typename varDouble, typename cvarDouble>
  inline void operator()(currDouble& pout, voltDouble* v, 
			 paramDouble* dparm, 
			 varDouble* tvar, cvarDouble* cvar)
  {
    const double &scale = options.getDHandler()->getParmVec()[OPT_SCALE];
    pout = v[0] * v[1] * scale;
  }

  // Adjust variables to a new temperature.
  template <typename tvarDouble, typename varDouble, typename tempDouble>
  inline void setTemp(tvarDouble* tvar, 
		      varDouble* dparm, varDouble* cvar, tempDouble& T)
  {
    // Do nothing here
  }
  
  // For efficiency, we may want to pre-calculate some expressions
  // that do not change with voltage or temperature.  Adjust variables
  // to a new parameer set.
  template <typename varDouble>
  inline void newParms(varDouble* cvar, varDouble* dparm)
  {
    // Do nothing here
  }

};


typedef FADBADGenericVCS<ThermalPowerS_eval, GenericVCCS, 2, 
			 MAX_DPARM_INDEX, TEMP, false> ThermalPowerS;

#endif


