// This may look like C code, but it is really -*- C++ -*-
//
// TimeDelayedCS is a class that handles current-sources controlled
// by time-delayed voltages:
//
//          2      
//          o                        o   o   o         o      
//          |                                                
//          |      	             +   +   +         +     
//         (^) i(v1,v2,v3,...,vn)    v1  v2  v3        vn
//          |      	             -   -   -         -     
//          |                                              
//          o      	             o   o   o         o     
//          1      
//
// a TimeDelayedCS have at least 4 terminals.
// 
// In the future this definitions should be made independent of
// GenericVCCS because the interface functions are not exactly equal,
// but for now it should work as is.

// by Carlos E. Christoffersen

#ifndef TimeDelayedCS_H
#define TimeDelayedCS_H 1

#include <carrot/GenericVCCS.h>

class TimeDelayedCS : public GenericVCCS
{
public:

  // ncvolt is the number of controlling voltages. Number of terminals
  // is: 2 output terminals + 2*number of controlling terminals.  
  TimeDelayedCS(const int& ncvolt) : GenericVCCS(ncvolt) 
  {
    // This element only has a meaning if there is at least one
    // controlling voltage.
    assert(ncvolt > 0);
  }
  
  virtual ~TimeDelayedCS() {}

  // Store port voltages at time t. This should be called at the end
  // of Newton iterations.
  virtual void storeVoltages(const double& t, double* v) {}

};


typedef std::vector<TimeDelayedCS*> TDCSVector;

#endif
