#include "tokenizer.h"
//#include <iostream>

static bool movePassedWhitespace(std::string::iterator & i, const std::string::iterator end)
{
  while (isblank(*i) && i != end){
    ++i;
  }
  return true;
}

static bool movePassedWhitespace(std::string::const_iterator & i, const std::string::const_iterator end)
{
  while (isblank(*i) && i != end){
    ++i;
  }
  return true;
}

static bool doesStartComment(std::string::const_iterator &i, const std::string::const_iterator end){
  //Is next major character an octothorp?
  if(*i == '#'){
    return true;
  }
  
  //Are next two characters double frontslashes?
  if(i[0] == '/' && (i+1) != end && i[1] == '/'){
    return true;
  }
  
  return false;
}

static bool doesStartComment(std::string::iterator &i, const std::string::iterator end){
  //Is next major character an octothorp?
  if(*i == '#'){
    return true;
  }

  //Are next two characters double frontslashes?
  if(i[0] == '/' && (i+1) != end && i[1] == '/'){
    return true;
  }

  return false;
}

bool tokenizeAnalysisType::isAnalysisType(const std::string & line){
  if(line.substr(0, 10) != ".analysis " ){
    return false;
  }

  //Add more tests here

  return true;
}

bool tokenizeOptionsType::isOptionsType(const std::string & line){
  if(line.substr(0, 9) != ".options " ){
    return false;
  }
  //Add more tests here
  return true;
}

bool tokenizeModuleDefinition::isModuleDefinition(const std::string & line){
  if(line.substr(0, 7) != ".model "){
    return false;
  }

  //Add more tests here to make sure that we are getting an appropriate line


  return true;
}

bool tokenizeSkipLine::isSkipLine(const std::string & line){
  std::string::const_iterator it = line.begin();

  movePassedWhitespace(it, line.end());

  //Is line all whitespace?
  if(it == line.end()){
    return true;
  }

  if(doesStartComment(it, line.end())){
    return true;
  }

  return false;
}

bool tokenizeEndOfFile::isEndOfFile(const std::string & line){
  if(line.substr(0,4) == ".end"){
    return true;
  }
  return false;
}

bool tokenizeCircuitItem::isCircuitItem(const std::string & line){
  if(line.substr(0, 7) == ".model "){
    return false;
  }

  //Add more tests here to make sure that we are getting an appropriate line

  return true;
}

void tokenizeModuleDefinition::parseString()
{
  //Clear out existing values

  elementName = "";
  instanceName = "";
  refModelName = "";
  parameters.clear();

  //Create an iterator which will be used to step through the string
  std::string::iterator it = parsedLine.begin();

  //Advance to avoid the ".model " bit;
  it += 7;
	
  //Get the element name
  movePassedWhitespace(it, parsedLine.end());
  while( *it != ':' && !isblank(*it) && it != parsedLine.end() ){
    elementName += *it;
    ++it;
  }

  movePassedWhitespace(it, parsedLine.end());
  if(*it == ':'){
    ++it;
  }
  movePassedWhitespace(it, parsedLine.end());


  //Now get the instance name
  while( *it != ':' && !isblank(*it) && it != parsedLine.end() ){
    instanceName += *it;
    ++it;
  }

  movePassedWhitespace(it, parsedLine.end());

  std::string part1;	//Used for parameter name
  std::string part2;	//User for parameter value


  //Now we process all remaining tokens involved

  while(it != parsedLine.end() && !doesStartComment(it, parsedLine.end())){
    part1 = "";
    part2 = "";
    while( *it != '=' && !isblank(*it) && it != parsedLine.end()){
      part1 += *it;
      ++it;
    }

    movePassedWhitespace(it, parsedLine.end());
		
    if(*it == '=' && it != parsedLine.end()){		//We have a parameter
      ++it;
      movePassedWhitespace(it, parsedLine.end());
      while( *it != '=' && !isblank(*it) && it != parsedLine.end()){
	part2 += *it;
	++it;
      }
      movePassedWhitespace(it, parsedLine.end());
      //Add the parameter
      if(part1 == "model"){
	refModelName = part2;
      }else{
	parameters[part1] = part2;
      }
    }else{
      //Just discard value
    }
		
  }
}



void tokenizeCircuitItem::parseString()
{
  //Clear out existing values

  elementName = "";
  instanceName = "";
  refModelName = "";
  parameters.clear();
  terminals.clear();

  //Create an iterator which will be used to step through the string
  std::string::iterator it = parsedLine.begin();

  //Get the element name
  movePassedWhitespace(it, parsedLine.end());
  while( *it != ':' && !isblank(*it) && it != parsedLine.end() ){
    elementName += *it;
    ++it;
  }

  movePassedWhitespace(it, parsedLine.end());
  if(*it == ':'){
    ++it;
  }
  movePassedWhitespace(it, parsedLine.end());


  //Now get the instance name
  while( *it != ':' && !isblank(*it) && it != parsedLine.end() ){
    instanceName += *it;
    ++it;
  }

  movePassedWhitespace(it, parsedLine.end());

  std::string part1;	//Used for parameter name
  std::string part2;	//User for parameter value


  //Now we process all remaining tokens involved

  while(it != parsedLine.end() && !doesStartComment(it, parsedLine.end())){
    part1 = "";
    part2 = "";
    while( *it != '=' && !isblank(*it) && it != parsedLine.end()){
      part1 += *it;
      ++it;
    }

    movePassedWhitespace(it, parsedLine.end());
		
    if(*it == '='){		//We have a parameter
      ++it;
      movePassedWhitespace(it, parsedLine.end());
      while( *it != '=' && !isblank(*it) && it != parsedLine.end()){
	part2 += *it;
	++it;
      }
      movePassedWhitespace(it, parsedLine.end());
      //Add the parameter
      if(part1 == "model"){
	refModelName = part2;
      }else{
	parameters[part1] = part2;
      }
    }else{		//We have a terminal name
      terminals.push_back(part1);
    }
  }
}

//Everything below here is less than one line of code and generally does not
//need to be looked at


tokenizeModuleDefinition::tokenizeModuleDefinition()
{

}

tokenizeModuleDefinition::tokenizeModuleDefinition(const std::string & line): 
  parsedLine(line)
{
  parseString();
}

tokenizeModuleDefinition::tokenizeModuleDefinition(const tokenizeModuleDefinition &t) :
  parsedLine(t.parsedLine), parameters(t.parameters), elementName(t.elementName),
  instanceName(t.instanceName), refModelName(t.refModelName)
{

}

tokenizeModuleDefinition::~tokenizeModuleDefinition()
{

}

void tokenizeModuleDefinition::setLineString(const std::string & s )
{
  parsedLine = s;
  parseString();
}

const std::string & tokenizeModuleDefinition::getElementName() const
{
  return elementName;
}

const std::string & tokenizeModuleDefinition::getInstanceName() const
{
  return instanceName;
}

const std::string & tokenizeModuleDefinition::getRefModelName() const
{
  return refModelName;
}

const std::map<std::string, std::string> & tokenizeModuleDefinition::getParameters() const
{
  return parameters;
}


tokenizeCircuitItem::tokenizeCircuitItem()
{

}

tokenizeCircuitItem::tokenizeCircuitItem(const std::string & line) :
  parsedLine(line)
{
  parseString();
}

tokenizeCircuitItem::tokenizeCircuitItem(const tokenizeCircuitItem &t) :
  parsedLine(t.parsedLine), parameters(t.parameters), terminals(t.terminals), 
  elementName(t.elementName), instanceName(t.instanceName), refModelName(t.refModelName)
{

}

tokenizeCircuitItem::~tokenizeCircuitItem()
{

}

void tokenizeCircuitItem::setLineString(const std::string & s )
{
  parsedLine = s;
  parseString();
}

const std::string & tokenizeCircuitItem::getElementName() const
{
  return elementName;
}

const std::string & tokenizeCircuitItem::getInstanceName() const
{
  return instanceName;
}

const std::string & tokenizeCircuitItem::getRefModelName() const
{
  return refModelName;
}

const std::map<std::string, std::string> & tokenizeCircuitItem::getParameters() const
{
  return parameters;
}

const std::vector<std::string> & tokenizeCircuitItem::getTerminals() const
{
  return terminals;
}


tokenizeAnalysisType::tokenizeAnalysisType()
{

}


tokenizeAnalysisType::tokenizeAnalysisType(const std::string & line) 
  : parsedLine(line)
{
  parseString();
}

tokenizeAnalysisType::~tokenizeAnalysisType()
{

}


void tokenizeAnalysisType::setLineString(const std::string & s )
{
  parsedLine = s;
  parseString();
}

const std::string & tokenizeAnalysisType::getAnalysisName() const{
  return analysisName;
}

const std::map<std::string, std::string> & tokenizeAnalysisType::getParameters() const{
  return parameters;
}

void tokenizeAnalysisType::parseString(){
  //Clear out existing values

  analysisName = "";
  parameters.clear();

  //Create an iterator which will be used to step through the string
  std::string::iterator it = parsedLine.begin();

  //Advance to avoid the ".analysis " bit;
  it += 10;
	
  //Get the element name
  movePassedWhitespace(it, parsedLine.end());
  while( *it != ':' && !isblank(*it) && it != parsedLine.end() ){
    analysisName += *it;
    ++it;
  }

  std::string part1;	//Used for parameter name
  std::string part2;	//User for parameter value


  //Now we process all remaining tokens involved

  while(it != parsedLine.end() && !doesStartComment(it, parsedLine.end())){
    part1 = "";
    part2 = "";
    while( *it != '=' && !isblank(*it) && it != parsedLine.end()){
      part1 += *it;
      ++it;
    }

    movePassedWhitespace(it, parsedLine.end());
		
    if(*it == '=' && it != parsedLine.end()){		//We have a parameter
      ++it;
      movePassedWhitespace(it, parsedLine.end());
      while( *it != '=' && !isblank(*it) && it != parsedLine.end()){
	part2 += *it;
	++it;
      }
      movePassedWhitespace(it, parsedLine.end());
      //Add the parameter
      parameters[part1] = part2;
    }else{
      //Just discard value
    }
		
  }
}

tokenizeOptionsType::tokenizeOptionsType()
{

}


tokenizeOptionsType::tokenizeOptionsType(const std::string & line) 
  : parsedLine(line)
{
  parseString();
}

tokenizeOptionsType::~tokenizeOptionsType()
{

}


void tokenizeOptionsType::setLineString(const std::string & s )
{
  parsedLine = s;
  parseString();
}

const std::map<std::string, std::string> & tokenizeOptionsType::getParameters() const{
  return parameters;
}

void tokenizeOptionsType::parseString(){
  //Clear out existing values
  parameters.clear();

  //Create an iterator which will be used to step through the string
  std::string::iterator it = parsedLine.begin();

  //Advance to avoid the ".options " bit;
  it += 9;
	
  std::string part1;	//Used for parameter name
  std::string part2;	//User for parameter value

  //Now we process all remaining tokens involved
  while(it != parsedLine.end() && !doesStartComment(it, parsedLine.end())){
    part1 = "";
    part2 = "";
    while( *it != '=' && !isblank(*it) && it != parsedLine.end()){
      part1 += *it;
      ++it;
    }

    movePassedWhitespace(it, parsedLine.end());
		
    if(*it == '=' && it != parsedLine.end()){		//We have a parameter
      ++it;
      movePassedWhitespace(it, parsedLine.end());
      while( *it != '=' && !isblank(*it) && it != parsedLine.end()){
	part2 += *it;
	++it;
      }
      movePassedWhitespace(it, parsedLine.end());
      //Add the parameter
      parameters[part1] = part2;
    }else{
      //Just discard value
    }
		
  }
}
