#ifndef TOKENIZER_H
#define TOKENIZER_H 1

#include <map>
#include <string>
#include <vector>



#endif

class tokenizeModuleDefinition{
  //This class parses a line from the input.  This should be replacable to support
  //any line-based grammar with very little effort.

 private:
  std::string parsedLine;
  std::map<std::string, std::string> parameters;
  std::string elementName;
  std::string instanceName;
  std::string refModelName;	//This is the name of the module we are copying existing settings from
	

  void parseString();

 public:

  static bool isModuleDefinition(const std::string & line);

  tokenizeModuleDefinition();
  tokenizeModuleDefinition(const std::string & line);
  tokenizeModuleDefinition(const tokenizeModuleDefinition &t);
  ~tokenizeModuleDefinition();

  void setLineString(const std::string & s );

  const std::string & getElementName() const;
  const std::string & getInstanceName() const;
  const std::string & getRefModelName() const;
  const std::map<std::string, std::string> & getParameters() const;

};


class tokenizeCircuitItem{
  //This class parses a line from the input.  This should be replacable to support
  //any line-based grammar with very little effort.

 private:
  std::string parsedLine;
  std::map<std::string, std::string> parameters;
  std::vector<std::string> terminals;
  std::string elementName;
  std::string instanceName;
  std::string refModelName;	//This is the name of the module we are copying existing settings from

  void parseString();

 public:

  static bool isCircuitItem(const std::string & line);

  tokenizeCircuitItem();
  tokenizeCircuitItem(const std::string & line);
  tokenizeCircuitItem(const tokenizeCircuitItem &t);
  ~tokenizeCircuitItem();

  void setLineString(const std::string & s );

  const std::string & getElementName() const;
  const std::string & getInstanceName() const;
  const std::string & getRefModelName() const;
  const std::map<std::string, std::string> & getParameters() const;
  const std::vector<std::string> & getTerminals() const;

};

class tokenizeSkipLine{
 private:

 public:
  static bool isSkipLine(const std::string & line);


};

class tokenizeEndOfFile{

 public:
  static bool isEndOfFile(const std::string & line);

};


class tokenizeAnalysisType{
 private:
  std::string parsedLine;
  std::map<std::string, std::string> parameters;
  std::string analysisName;

  void parseString();

 public:

  static bool isAnalysisType(const std::string & line);

  tokenizeAnalysisType();
  tokenizeAnalysisType(const std::string & line);
  ~tokenizeAnalysisType();

  void setLineString(const std::string & s );

  const std::string & getAnalysisName() const;
  const std::map<std::string, std::string> & getParameters() const;
};

class tokenizeOptionsType{
 private:
  std::string parsedLine;
  std::map<std::string, std::string> parameters;

  void parseString();

 public:

  static bool isOptionsType(const std::string & line);

  tokenizeOptionsType();
  tokenizeOptionsType(const std::string & line);
  ~tokenizeOptionsType();

  void setLineString(const std::string & s );

  const std::map<std::string, std::string> & getParameters() const;
};

